///
/// @file
/// Declares macros for the cpp-hocon library version.
///
#pragma once

///
/// The cpp-hocon library major version.
///
#define CPP_HOCON_VERSION_MAJOR 0
///
/// The cpp-hocon library minor version.
///
#define CPP_HOCON_VERSION_MINOR 3
///
/// The cpp-hocon library patch version.
///
#define CPP_HOCON_VERSION_PATCH 0

///
/// The cpp-hocon library version as a string (without commit SHA1).
///
#define CPP_HOCON_VERSION "0.3.0"

///
/// The cpp-hocon library version as a string (with commit SHA1).
///
#define CPP_HOCON_VERSION_WITH_COMMIT "0.3.0"
